/*
	"Under Pressure"
		by natrium42

	Using joat's template and DesktopMan's tetris example as a base for this file.

	Thanks to:
		+ joat and dovoto for great research and ndslib
		+ DarkFader for realizing the passthrough method
		+ DesktopMan for writing Tetris and releasing code
		+ All other developers making all this possible

	Feel free to modify this project :)
*/


// includes from ndslib
#include <NDS/NDS.h>
#include <NDS/ARM9/video.h>
#include <NDS/memory.h>

// includes from libdesk
#include <ipc.h>
#include <vblank.h>
#include <mt.h>
#include <text.h>
#include <bg.h>

// project includes
#include "global.h"

// Data headers
#include "font_raw.h"
#include "font_pal.h"

#include "back_pal.h"

#include "logo_raw.h"
#include "logo_map.h"

#include "powerbar_raw.h"
#include "powerbar_map.h"

#define TRANSPARENT			0

int16 currX = 96;
int16 currY = 0;
int16 speedX = 1;
int16 speedY = 2;

//////////////////////////////////////////////////////////////////////

inline void COPY16(u16* from,u16* to,int size) {
	for(int i=0;i<size;i++)
		to[i]=from[i];
}

//////////////////////////////////////////////////////////////////////

void init(cBgBase* powerbar, cBgBase* logo, cBgBase* text) {
	// All tiles transparent at startup, both font data and tile data has pos 0 as transparent.
	for(int pos = 0; pos < 1024; pos++) {
		text->map[pos] = TRANSPARENT;
		//logo->map[pos] = ((uint16*)logo_map)[pos];
		//powerbar->map[pos] = ((uint16*)powerbar_map)[pos];
		powerbar->map[pos] = TRANSPARENT;
	}
	for(int pos = 0; pos < 4096; pos++) {
		/*
		uint16 x = pos % 64;
		uint16 y = pos / 64;
		if(x < 32 && y < 24)
			logo->map[pos] = ((uint16*)logo_map)[y * 32 + x];
		else
			logo->map[pos] = TRANSPARENT;
		*/
		logo->map[pos] = ((uint16*)logo_map)[pos];
	}
}

//////////////////////////////////////////////////////////////////////

void drawBars(cBgBase* powerbar, uint16 num) {
	// x = 12, y = 9
	// width = 8, height = 5;

	// first draw grey powerbars:
	for(uint16 x = 0; x < 8; x++) {
		for(uint16 y = 0; y < 5; y++) {
			uint16 tileScreen = (y + 9) * 32 + (x + 12);
			uint16 tileMap = y * 32 + x;
			powerbar->map[tileScreen] = ((uint16*)powerbar_map)[tileMap];
		}
	}

	// then draw colorful powerbars:
	for(uint16 x = 0; x < num; x++) {
		for(uint16 y = 0; y < 5; y++) {
			uint16 tileScreen = (y + 9) * 32 + (x + 12);
			uint16 tileMap = y * 32 + (x + 8);
			powerbar->map[tileScreen] = ((uint16*)powerbar_map)[tileMap];
		}
	}
	
}

//////////////////////////////////////////////////////////////////////

void animateBackground(void) {
	currX += speedX;
	currY += speedY;

	if(currX == 0 || currX == 256 - 64)
		speedX = -speedX;
	if(currY == 0 || currY == 192 - 64)
		speedY = -speedY;



	BG1_X0 = -(currX - 224);
	SUB_BG1_X0 = -(currX - 224);

	BG1_Y0 = -(currY - 224);
	SUB_BG1_Y0 = -(currY - 224);
}

//////////////////////////////////////////////////////////////////////

int main() {
	POWER_CR = POWER_ALL_2D;

	VRAM_A_CR = VRAM_ALL_ENABLE | VRAM_ABCD_OBJ;
	VRAM_B_CR = VRAM_ALL_ENABLE | VRAM_ABCD_BG;

	VRAM_C_CR = VRAM_ALL_ENABLE | VRAM_CORE_B;
	VRAM_D_CR = VRAM_ALL_ENABLE | VRAM_CORE_B;

	// Wait before doing alot of stuff
	WaitForVblank();

	// Text bg, 16 color, char base 1 and screen base 0
	cBg texta(0, BG_COLOR_16, 1, 0);
	// Copy font data to vram
	COPY16((u16*)font_raw, texta.tile, (int) font_raw_size >> 1);

	// logo bg, 16 color, char base 2 and screen base 1
	cBg logoa(1, BG_COLOR_256 | BG_PRIORITY_3 | BG_64x64 | BG_WRAP_ON, 2, 1);
	// Copy font data to vram
	COPY16((u16*)logo_raw, logoa.tile, (int) logo_raw_size >> 1);

	// powerbar bg, 16 color, char base 3 and screen base 2
	cBg powerbara(2, BG_COLOR_256 | BG_PRIORITY_0, 3, 5);
	// Copy font data to vram
	COPY16((u16*)powerbar_raw, powerbara.tile, (int) powerbar_raw_size >> 1);

	// Text bg, 16 color, char base 1 and screen base 0
	cBgSub textb(0, BG_COLOR_16, 1, 0);
	// Copy font data to vram
	COPY16((u16*)font_raw, textb.tile, (int) font_raw_size >> 1);

	// logo bg, 16 color, char base 2 and screen base 1
	cBgSub logob(1, BG_COLOR_256 | BG_PRIORITY_3 | BG_64x64 | BG_WRAP_ON, 2, 1);
	// Copy font data to vram
	COPY16((u16*)logo_raw, logob.tile, (int) logo_raw_size >> 1);

	// powerbar bg, 16 color, char base 3 and screen base 2
	cBgSub powerbarb(2, BG_COLOR_256 | BG_PRIORITY_0, 3, 5);
	// Copy font data to vram
	COPY16((u16*)powerbar_raw, powerbarb.tile, (int) powerbar_raw_size >> 1);

	cBgBase *text[2];
	text[0] = &texta;
	text[1] = &textb;

	cBgBase *logo[2];
	logo[0] = &logoa;
	logo[1] = &logob;

	cBgBase *powerbar[2];
	powerbar[0] = &powerbara;
	powerbar[1] = &powerbarb;

	init(powerbar[0], logo[0], text[0]);
	init(powerbar[1], logo[1], text[1]);

	// Copy palettes to vram
	COPY16((u16*)font_pal, &BGPaletteMem[256-16], 16);
	COPY16((u16*)font_pal, &BGPaletteMemSub[256-16], 16);
	COPY16((u16*)back_pal, &BGPaletteMem[0], 256-16);
	COPY16((u16*)back_pal, &BGPaletteMemSub[0], 256-16);

	// Black backdrop
	BGPaletteMem[0] = 0x0;
	BGPaletteMemSub[0] = 0x0;

	// Wait until vblank before turning the backgrounds on.
	WaitForVblank();

	// Text mode, bgs on.
	DISPLAY_CR	   = MODE_0_2D | DISPLAY_BG0_ACTIVE | DISPLAY_BG1_ACTIVE | DISPLAY_BG2_ACTIVE;
	SUB_DISPLAY_CR = MODE_0_2D | DISPLAY_BG0_ACTIVE | DISPLAY_BG1_ACTIVE | DISPLAY_BG2_ACTIVE;

	TextWrite(8, 1, "\"Under Pressure\"", text[1]);
	TextWrite(8, 3, "  by natrium42", text[1]);

	TextWrite(8, 1, "Touching is good", text[0]);
	
	TextWrite(6, 22, "Greetings to #dsdev!", text[0]);


	drawBars(powerbar[0], 2);
	drawBars(powerbar[1], 5);
	

	// Main game loop
	while(1) {
		WaitForVblank();
		animateBackground();

		// here we calculate the (pseudo) pressure:

		uint16 touchZ1 = IPC->touchZ1;
		uint16 touchZ2 = IPC->touchZ2;
		uint16 touchX = IPC->touchX;

		uint16 pressure = (touchX * touchZ2) / (64 * touchZ1) - touchX / 64;

		// range is about 0 to 128 (with the given divisor of 64 above)
		// pressure of stylus is around 100
		// pressure of finger is around 40

		if(pressure < 128) {
			TextWrite(14, 15, "         ", text[0]);
			TextWrite(14, 15, pressure, text[0]);

			drawBars(powerbar[0], pressure * 8 / 128);
			drawBars(powerbar[1], pressure * 8 / 128);
		}

		// delay loop:
		for(uint16 i = 0; i < 8; i++) {
			WaitForVblank();
			animateBackground();
		}
	}
}

